package com.hai.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.config.CommonSysConst;
import com.hai.config.JinZhuJiaYouService;
import com.hai.config.ShellGroupService;
import com.hai.config.TuanYouConfig;
import com.hai.entity.*;
import com.hai.enum_type.*;
import com.hai.msg.entity.MsgTopic;
import com.hai.order.service.OrderPaySuccessService;
import com.hai.order.service.OrderService;
import com.hai.order.topic.OrderTopic;
import com.hai.order.type.OrderChildGoodsType;
import com.hai.order.type.OrderChildStatus;
import com.hai.order.type.OrderPayType;
import com.hai.order.type.OrderStatus;
import com.hai.service.*;
import org.apache.rocketmq.client.producer.SendCallback;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.integration.support.MessageBuilder;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @className: OrderPaySuccessServiceImpl
 * @author: HuRui
 * @date: 2022/8/29
 **/
@Service("orderPaySuccessService")
public class OrderPaySuccessServiceImpl implements OrderPaySuccessService {

    @Resource
    private RedisTemplate redisTemplate;

    @Resource
    private RocketMQTemplate rocketMQTemplate;

    @Resource
    private OrderService orderService;

    @Resource
    private HighOrderPreService orderPreService;

    @Resource
    private HighMerchantStoreService merchantStoreService;

    @Resource
    private HighCouponService couponService;

    @Resource
    private HighCouponCodeService couponCodeService;

    @Resource
    private HighUserService userService;

    @Resource
    private HighGasOrderService gasOrderService;

    @Resource
    private ShellGroupService shellGroupService;

    @Resource
    private JinZhuJiaYouService jinZhuJiaYouService;

    @Override
    public HighOrder orderPaySuccessHandle(String orderNo, OrderPayType payType, String paySerialNo, BigDecimal payRealPrice, HighUserCard userCard) {
        HighOrder order = orderService.getOrderDetailByNo(orderNo);
        try {
            if (order != null && order.getOrderStatus().equals(OrderStatus.STATUS2.getNumber())) {
                order.setPaySerialNo(paySerialNo);
                order.setPayRealPrice(payRealPrice);
                order.setPayTime(new Date());
                order.setPayType(payType != null?payType.getNumber():null);
                order.setOrderStatus(OrderStatusEnum.type2.getType());
                if (userCard != null) {
                    order.setMemCardId(userCard.getId());
                    order.setMemCardNo(userCard.getCardNo());
                    order.setMemCardType(userCard.getType());
                }
                for (HighChildOrder childOrder : order.getHighChildOrderList()) {
                    childOrder.setPayTime(new Date());
                    childOrder.setChildOrderStatus(OrderChildStatus.STATUS2.getNumber());
                }
                orderService.updateOrderData(order);

                // 分账业务
                if (order.getProfitSharingStatus() == true) {
                    // 延迟3分钟
                    Message<HighOrder> rocketMsg = MessageBuilder.withPayload(order).build();
                    rocketMQTemplate.syncSend(OrderTopic.ORDER_TOPIC_PROFIT_SHARING.getTopic(), rocketMsg, 1000,7);
                }

                // 处理订单产品业务
                orderBusHandle(order);
            }
            return order;

        } catch (Exception e) {
            if (order != null) {
                order.setExceptionStatus(true);
                order.setExceptionContent(e.getMessage());
                orderService.updateOrderData(order);
            }
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, e.getMessage());
        }
    }

    /**
     * 订单业务处理
     * @param order
     * @throws Exception
     */
    @Transactional(
            propagation= Propagation.REQUIRES_NEW,
            isolation = Isolation.READ_COMMITTED,
            timeout = 10,
            rollbackFor = Exception.class)
    public void orderBusHandle(HighOrder order) throws Exception {
        for (HighChildOrder childOrder : order.getHighChildOrderList()) {
            if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE1.getNumber())) {
                couponHandle(order, childOrder);

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE2.getNumber())) {
                integralHandle(order,childOrder);

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE3.getNumber())) {
                oilHandle(order,childOrder);

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE4.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE5.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE6.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE7.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE8.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE9.getNumber())) {

            } else if (childOrder.getGoodsType().equals(OrderChildGoodsType.TYPE10.getNumber())) {

            }
        }
    }

    /**
     * 卡券业务
     * @param order
     * @param childOrder
     * @throws Exception
     */
    public void couponHandle(HighOrder order,HighChildOrder childOrder) throws Exception {
        HighCoupon coupon = couponService.getCouponById(childOrder.getGoodsId());
        // 是否预约
        if (coupon.getReserveStatus()) {
            HighOrderPre orderPre = new HighOrderPre();
            orderPre.setCompanyId(coupon.getCompanyId());
            orderPre.setMerchantId(coupon.getMerchantId());
            orderPre.setMerchantStoreId(childOrder.getStoreId());
            orderPre.setPreOrderNo(System.currentTimeMillis()+"");
            orderPre.setOrderId(order.getId());
            orderPre.setOrderNo(order.getOrderNo());
            orderPre.setChildOrderId(childOrder.getId());
            orderPre.setPayRealPrice(order.getPayRealPrice());
            orderPre.setGoodsType(1);
            orderPre.setGoodsName(coupon.getCouponName());
            orderPre.setGoodsId(coupon.getId());
            orderPre.setPreUserId(order.getMemId());
            orderPre.setPreUserName(order.getMemName());
            orderPre.setPreUserPhone(order.getMemPhone());
            orderPre.setPreUserRemark(order.getRemarks());
            orderPre.setStatus(1);
            orderPreService.insertOrderPre(orderPre);
        }

        // 支付购买卡券
        couponCodeService.payCoupon(order,childOrder,coupon);

    }

    /**
     * 积分充值业务
     * @param order
     * @param childOrder
     */
    public void integralHandle(HighOrder order,HighChildOrder childOrder) {
        // 金币 1：100
        Integer goldNum = new BigDecimal(childOrder.getTotalPrice().toString()).multiply(new BigDecimal("100")).intValue();
        String remark = "订单" + order.getOrderNo() + "积分充值积分：" +  goldNum;
        userService.goldHandle(childOrder.getGoodsId(), goldNum, 1, 1, childOrder.getId() , remark);
    }

    /**
     * 加油业务
     * @param order
     * @param childOrder
     */
    public void oilHandle(HighOrder order,HighChildOrder childOrder) throws Exception {
        // 查询加油订单
        HighGasOrder gasOrder = gasOrderService.getDetailByChildOrderNo(childOrder.getChildOrderNo());
        if (gasOrder != null) {
            gasOrder.setPayTime(new Date());
            gasOrder.setStatus(OrderChildStatus.STATUS2.getNumber());
            gasOrderService.updateGasOrder(gasOrder);

            // 查询油站
            HighMerchantStore store = merchantStoreService.getDetailById(gasOrder.getStoreId());
            if (store == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到门店");
            }
            if (gasOrder.getChannelType().equals(MerchantStoreSourceType.type1.getNumber())) {
                // 预存类型  0：非预存  1：预存门店
                if (store.getPrestoreType() != null && store.getPrestoreType().equals(1)) {
                    Map<String, Object> pushParam = new HashMap<>();
                    pushParam.put("businessType", MerStoreAmountTypeEnum.type2.getType());
                    pushParam.put("storeId", childOrder.getGoodsId());
                    pushParam.put("price", order.getTotalPrice());
                    pushParam.put("sourceType", MerStoreAmountSourceTypeEnum.type2.getType());
                    pushParam.put("sourceId", order.getId());
                    pushParam.put("sourceContent", "订单号：" + order.getOrderNo() + ",加油金额：￥" + order.getTotalPrice());
                    pushParam.put("opUserId", order.getMemId());
                    pushParam.put("opUserName", order.getMemName());
                    // 扣预存款
                    this.redisTemplate.boundListOps(MsgTopic.MerStoreAccount.getName()).leftPush(pushParam);
                }

            } else if (gasOrder.getChannelType().equals(MerchantStoreSourceType.type2.getNumber())) {
                // 推送团油订单
                Map<String, Object> paramMap = new HashMap<>();
                paramMap.put("gasId", store.getStoreKey());
                paramMap.put("oilNo", gasOrder.getGasOilNo());
                paramMap.put("gunNo", gasOrder.getGasGunNo());
                paramMap.put("priceGun", gasOrder.getGasPriceGun());  // 枪单价
                paramMap.put("priceVip", gasOrder.getGasPriceVip());  // 优惠价
                paramMap.put("driverPhone", gasOrder.getMemPhone());
                paramMap.put("thirdSerialNo", gasOrder.getOrderNo());
                paramMap.put("refuelingAmount", gasOrder.getGasRefuelPrice());

                // 油品类型 1:汽油:2:柴油;3:天然气
                if (gasOrder.getGasOilType() == 1) {
                    paramMap.put("accountNo", CommonSysConst.getSysConfig().getTuanYouGasolineAccount());
                } else if (gasOrder.getGasOilType() == 2) {
                    paramMap.put("accountNo", CommonSysConst.getSysConfig().getTuanYouDieselAccount());
                }
                JSONObject orderPushObject = TuanYouConfig.refuelingOrderPush(paramMap);
                if (orderPushObject != null && orderPushObject.getString("code").equals("200")) {
                    gasOrder.setChannelOrderNo(orderPushObject.getJSONObject("result").getString("orderNo"));
                }

            } else if (gasOrder.getChannelType().equals(MerchantStoreSourceType.type3.getNumber())) {
                // 推送加好油
                shellGroupService.gasSyncPayment(gasOrder.getOrderNo(),
                        store.getStoreKey(),
                        gasOrder.getPayTime(),
                        order.getTotalPrice(),
                        gasOrder.getGasOilNo(),
                        gasOrder.getGasGunNo(),
                        order.getPayablePrice(),
                        order.getDeductionPrice()
                );

            } else if (gasOrder.getChannelType().equals(MerchantStoreSourceType.type4.getNumber())) {
                // 推送金猪加油
                jinZhuJiaYouService.payOrder(order.getOrderNo(), gasOrder.getGasPriceChannelPay());

            }
        }
    }

}
