package com.v1.config;


import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpQrCodeTicket;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;

/**
 * @serviceName WeChatQrcodeUtils.java
 * @author Sum1Dream
 * @version 1.0.0
 * @Description // 生成带参数的二维码
 * @createTime 16:11 2022/9/2
 **/
@Slf4j
@Component
public class WeChatQrcodeUtils {

    @Autowired
    private WxMpService wxMpService;

    /**
     * <pre>
     * 创建临时二维码ticket
     * @param sceneId       场景值ID，临时二维码时为32位非0整型
     * @param expireSeconds 该二维码有效时间，以秒为单位。 最大不超过2592000（即30天），此字段如果不填，则默认有效期为30秒。
     * @auther: cao_wencao
     * @date: 2019/2/18 16:58
     * </pre>
     */
    public WxMpQrCodeTicket qrCodeCreateTmpTicket(int sceneId, Integer expireSeconds) throws WxErrorException {
        WxMpQrCodeTicket wxMpQrCodeTicket = wxMpService.getQrcodeService().qrCodeCreateTmpTicket(sceneId, expireSeconds);
        return wxMpQrCodeTicket;
    }

    /**
     * <pre>
     * 创建临时二维码ticket
     * @param sceneStr      场景值ID（字符串形式的ID），字符串类型，长度限制为1到64
     * @param expireSeconds 该二维码有效时间，以秒为单位。 最大不超过2592000（即30天），此字段如果不填，则默认有效期为30秒。
     * @auther: cao_wencao
     * @date: 2019/2/18 17:01
     * </pre>
     */
    public WxMpQrCodeTicket qrCodeCreateTmpTicket(String sceneStr, Integer expireSeconds) throws WxErrorException {
        WxMpQrCodeTicket wxMpQrCodeTicket = wxMpService.getQrcodeService().qrCodeCreateTmpTicket(sceneStr, expireSeconds);
        return wxMpQrCodeTicket;
    }

    /**
     * <pre>
     * 创建永久二维码ticket
     * @param sceneId 场景值ID，最大值为100000（目前参数只支持1--100000）
     * @auther: cao_wencao
     * @date: 2019/2/18 17:03
     * </pre>
     */
    public WxMpQrCodeTicket qrCodeCreateLastTicket(int sceneId) throws WxErrorException {
        return wxMpService.getQrcodeService().qrCodeCreateLastTicket(sceneId);
    }

    /**
     * <pre>
     * 创建永久字符串二维码ticket
     * @param sceneStr 参数。字符串类型长度现在为1到64
     * @auther: cao_wencao
     * @date: 2019/2/18 17:05
     * </pre>
     */
    public WxMpQrCodeTicket qrCodeCreateLastTicket(String sceneStr) throws WxErrorException {
        return wxMpService.getQrcodeService().qrCodeCreateLastTicket(sceneStr);
    }

    /**
     * <pre>
     * 换取二维码图片文件，jpg格式
     * @param ticket 二维码ticket
     * @auther: cao_wencao
     * @date: 2019/2/18 17:07
     * </pre>
     */
    public File qrCodePicture(WxMpQrCodeTicket ticket) throws WxErrorException {
        return wxMpService.getQrcodeService().qrCodePicture(ticket);

    }

    /**
     * <pre>
     * 换取二维码图片url地址（可以选择是否生成压缩的网址）
     * @param ticket       二维码ticket
     * @param needShortUrl 是否需要压缩的二维码地址
     * @auther: cao_wencao
     * @date: 2019/2/18 17:10
     * </pre>
     */
    public String qrCodePictureUrl(String ticket, boolean needShortUrl) throws WxErrorException {
        String qrCodeUrl = wxMpService.getQrcodeService().qrCodePictureUrl(ticket, needShortUrl);
        return qrCodeUrl;
    }

    /**
     * <pre>
     * 换取二维码图片url地址
     * @param ticket 二维码ticket
     * @auther: cao_wencao
     * @date: 2019/2/18 17:11
     * </pre>
     */
    public String qrCodePictureUrl(String ticket) throws WxErrorException {
        String url = wxMpService.getQrcodeService().qrCodePictureUrl(ticket);
        return url;
    }


}
